(** Time-stamp: <modified the 28/05/2008 (at 14:08) by Erwan Jahier> *)


open Lxm
open Errors
open SyntaxTree
open SyntaxTreeCore

(* get the first package in the package/model list *)
let rec first_pack_in =
  function
    | (NSPack pi)::_ -> pi.it.pa_name
    | (NSModel _)::tail -> first_pack_in tail
    | [] -> raise (Global_error "No package has been provided")

open Global

let (doit : SyntaxTree.pack_or_model list -> Ident.idref -> unit) = 
  fun srclist main_node -> 

  let syntax_tab = SyntaxTab.create srclist in
    if Verbose.get_level () > 2 then SyntaxTab.dump syntax_tab;

    Ident.set_dft_pack_name (first_pack_in srclist);

    (* la cle "absolue" du main node (pas d'args statiques) *)
    let main_node_key = 
      CompiledData.make_simple_node_key (Ident.long_of_idref main_node) 
    in

      Verbose.printf 
	"-- MAIN NODE: \"%s\"\n" 
	(CompiledDataDump.string_of_node_key main_node_key);

      (* Pour chaque package, on a un solveur de rfrences
	 globales, pour les types, const et node :
	 - les rfrences pointes (p::n) sont recherches
	 directement dans la syntax_tab puisqu'il n'y a pas 
	 d'ambiguit
	 - les rfrences simples sont recherches :
         . dans le pack lui-mme
         . dans un des packs dclars "uses", avec
         priorit dans l'ordre
      *)
      let lzcomp = LazyCompiler.create syntax_tab in
	if Global.vars.compile_all_items then
	  LazyCompiler.compile_all lzcomp
	else
	  ignore(LazyCompiler.node_check lzcomp main_node_key 
	    (match Ident.pack_of_idref main_node with 
	       | None -> Lxm.dummy "" 
	       | Some pn  -> Lxm.dummy (Ident.pack_name_to_string pn)))
